import sys
from ._player import *
from ._network import *
from ._game import *

player_types = [
    ('computer players', ComputerPlayer),
    ('real players (1 or 0)', RealPlayer),
    ('LAN players', NetworkPlayer),
]

def get_number(value):
        try: value = int(value)
        except ValueError:
            print('Please enter a number.')
            return
        else: return value

def play_game(quitter):
    player_list = []
    for (label, klass) in player_types:
        value = get_number(input('How many %s would you like? ' % label))
        while value is None:
            value = get_number(input('How many %s would you like? ' % label))
        for _ in range(value):
            player_list.append(klass())
    game = Game(player_list)
    game.begin()

def join_game(quitter):
    client = GameClient(RealPlayer())
    while True:
        host = input('Hostname/IP of the game to join: ')
        try: client.connect((host, 8660))
        except socket.error:
            print('Unable to connect to game')
            continue
        else: break
    while True:
        try: client.poll()
        except GameOver: break
        except ConnectionError:
            print('Disconnected from game host.')
            break

def end(quitter):
    print('Goodbye.')
    quitter[0] = True

options = [
    ('Play Game', play_game),
    ('Join Network Game', join_game),
    ('Quit', end),
]

def opt_display(name, opts):
    value = name
    for (n, (label, func)) in enumerate(opts):
        value += '\t%i:%s' % (n, label)
    return value

def menu(name, options):
    quitter = [False]
    while True:
        print(opt_display(name, options))
        value = input('Which option: ')
        value = get_number(value)
        if value is None: continue
        elif value < len(options):
            options[value][1](quitter)
        else: print('Invalid option: %s' % value)
        if quitter[0]: return

def exit_callback(): pass

def main():
    if '-V' in sys.argv or '--version' in sys.argv:
        from ._mods import __version__
        print('Uno', __version__)
        sys.exit()
    try: menu('Main Menu', options)
    except KeyboardInterrupt: pass
    exit_callback()

# __all__ = dir()

if __name__ == '__main__': main()


class Color:
    def __init__(self, code, name, ansi):
        self.code = code
        self.name = name
        self.ansi = '3' + str(ansi)
    def __eq__(self, other):
        return (
            self.code == other.code and
            self.name == other.name
        )
    def __hash__(self):
        return hash(self.code) * 1 + hash(self.name) * 8

# __all__ = dir()


from netsc import Client
from .other import deserialize_card, serialize_card, GameSubstitute, serialize_color, deserialize_color
from .. import Color

class GameClient(Client):
    def deadj_args_start(self, *args):
        return (GameSubstitute(self),), {}
    def deadj_args_play(self, *args):
        return (deserialize_card(args[0]), GameSubstitute(self)), {}
    def adj_return_play(self, card):
        if card is None:
            return card
        return serialize_card(card)
    def deadj_args_ask(self, q, t, limits=()):
        return (q, getattr(__import__(t[0]), t[1])), {'limits': [deserialize_color(l) for l in limits]}
    def adj_return_ask(self, value):
        if isinstance(value, Color):
            return serialize_color(value)
        return value
    def deadj_args_can_play_card(self, card1, card2):
        return (deserialize_card(card1), deserialize_card(card2)), {}
    def deadj_args_can_play(self, card):
        return (deserialize_card(card),), {}
    def deadj_args_remove_from_hand(self, card):
        return (deserialize_card(card),), {}
    def adj_return___getattribute__(self, value):
        if value is self.wrapped.hand:
            return [serialize_card(x) for x in value]
        else: return value
    def post_call_end(self):
        raise GameOver

class GameOver(BaseException): pass


from .. import CARD_SET, _colors, Wild, copy

COLOR_SET = {getattr(_colors, c) for c in dir(_colors) if isinstance(getattr(_colors, c), _colors.Color)}
WILD_SET = set()
color_list = list(COLOR_SET)
for card in CARD_SET:
    if isinstance(card, Wild):
        temp_list = (copy.copy(card) for i in range(len(color_list)))
        for (i, subcard) in enumerate(temp_list):
            subcard.color = color_list[i]
            WILD_SET.add(subcard)
del color_list, card, temp_list, i, subcard

def serialize_card(card):
    return (card.__class__.__name__, card.short_name, card.color.name)

def deserialize_card(data):
    for card in CARD_SET | WILD_SET:
        if data == serialize_card(card):
            return card
    print('ERROR: could not deserialize card, substituting random card')
    return card

def serialize_color(color):
    return (color.code, color.name)

def deserialize_color(data):
    for color in COLOR_SET:
        if data == serialize_color(color):
            return color
    print('ERROR: could not deserialize color, substituting random color')
    return color

class GameSubstitute:
    def __init__(self, client_obj):
        self.client_obj = client_obj
    def __getattr__(self, attr):
        return getattr(self.client_obj, attr)


from .server import GameServer
from .client import GameClient, GameOver
from . import shared

def NetworkPlayer(card_count=7):
    server = GameServer(None)
    print('Waiting for Player', shared.player_count+1, 'to join...', end='\r')
    addr = server.accept()
    player_name = server.name
    print('%s joined the game!' % player_name, ' '*(11+len(player_name)))
    shared.player_count += 1
    return server


import setuptools

setuptools.setup(
    name = 'Uno Game',
    version = '1.1b12',
    url = 'https://github.com/gaming32/uno',
    author = 'Gaming32',
    author_email = 'gaming32i64@gmail.com',
    license = 'License :: OSI Approved :: MIT License',
    description = 'A recreation of the Uno card game',
    long_description = open('README.md').read(),
    long_description_content_type = 'text/markdown',
    install_requires = [
        'numpy',
        'Network-Script>=1.0.5',
        'Roman-Numerals-Simple',
    ],
    packages = [
        'uno',
        'uno._network',
    ],
)


_rom_val = [
    1000, 900, 500, 400,
    100, 90, 50, 40,
    10, 9, 5, 4,
    1
]
_rom_syb = [
    "M", "CM", "D", "CD",
    "C", "XC", "L", "XL",
    "X", "IX", "V", "IV",
    "I"
]
def int_to_roman(num):
    roman_num = ''
    i = 0
    while num > 0:
        for _ in range(num // _rom_val[i]):
            roman_num += _rom_syb[i]
            num -= _rom_val[i]
        i += 1
    return roman_num

questions = {
    'wild': "What would you like to change the color to? ",
    'call wild': "The last player played a Draw Four Wild. Would you like to call? "
}
